# !/usr/bin/env python
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Title       : utils.py
# Author      : aDd1kTeD2Ka0s (add1kted2ka0s@gmail.com)
# Date        : 20-Jun-2014
# Description : Ka0s Leaderboard - Utilities
#             : Original code by Rombik
# -----------------------------------------------------------------------------

# -------------------------------------
# Imports
# -------------------------------------

import ac
import functools
import threading
import traceback

# -------------------------------------
class Signal(object):
	# -------------------------------------
	# Lightweight implementation of Qt Signal Slot for pure Python objects.

	# -------------------------------------
	def __init__(self, *types):
		# -------------------------------------
		self.slots = []
		self.types = types  # not really used

	# -------------------------------------
	def connect(self, callable_slot):
		# -------------------------------------
		self.slots.append(callable_slot)

	# -------------------------------------
	def emit(self, *args, **kwargs):
		# -------------------------------------
		for slot in self.slots:
			slot(*args, **kwargs)

	# -------------------------------------
	def disconnect(self, callable_slot):
		# -------------------------------------
		try:
			self.slots.remove(callable_slot)
		except ValueError:
			pass

# -------------------------------------
def async(func):
	# -------------------------------------
	@functools.wraps(func)
	def wrapper(*args, **kwargs):
		t = threading.Thread(target=func, args=args, kwargs=kwargs)
		t.daemon = True
		t.start()
		return t

	return wrapper

# -------------------------------------
def log(s):
	# -------------------------------------
	msg = 'KLB>{}'.format(s)
	ac.log(msg)
	ac.console(msg)

# -------------------------------------
def funclog(func):
	# -------------------------------------
	@functools.wraps(func)
	def wrapper(*args, **kwargs):
		log('Calling: %s(args=%s kwargs=%s)' % (func.__name__, args, kwargs))
		try:
			return func(*args, **kwargs)
		except Exception:
			log_traceback()
			raise

	return wrapper

# -------------------------------------
def with_lock(lock_group, lock_class=threading.Lock):
	# -------------------------------------
	registry = getattr(with_lock, '_locks_registry', None)
	if registry is None:
		registry = with_lock._locks_registry = {}
	lock = registry.get(lock_group, None)
	if lock is None:
		lock = registry[lock_group] = lock_class()

	def decorator(func):
		@functools.wraps(func)
		def wrapper(*args, **kwargs):
			with lock:
				return func(*args, **kwargs)

		return wrapper

	return decorator

# -------------------------------------
def log_traceback():
	# -------------------------------------
	msg = 'Exception: %s' % traceback.format_exc()
	log(msg)
