# !/usr/bin/env python
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Title       : widgets.py
# Author      : aDd1kTeD2Ka0s (add1kted2ka0s@gmail.com)
# Date        : 20-Jun-2014
# Description : Ka0s Leaderboard - Widgets
# 			  : Original author - @softgrip
# 			  : http://www.assettocorsa.net/forum/index.php?threads/rivali-tempo-4-4.3215/
# -----------------------------------------------------------------------------

# -------------------------------------
# Imports
# -------------------------------------

import ac

from .colors import *

# -------------------------------------
# Globals
# -------------------------------------

scale_by = 1.0
XY = namedtuple('XY', 'x y')

# -------------------------------------
def scale(n):
	# -------------------------------------
	return round(n * scale_by)

# -------------------------------------
class Control():
	# -------------------------------------

	# -------------------------------------
	def __init__(self, app_id):
		# -------------------------------------
		self.app_id = app_id
		self.opacity = 0.1
		self.border = 0
		self.size = XY(0, 0)
		self.alpha = 1
		self.rgb = RGB(0, 0, 0)
		self.pos = XY(0, 0)
		self.is_visible = True
		self.children = None

	# -------------------------------------
	def refresh_opacity(self):
		# -------------------------------------
		self.set_bg_opacity(self.opacity)
		self.set_border(self.border)
		return self

	# -------------------------------------
	def add_click_listener(self, func):
		# -------------------------------------
		ac.addOnClickedListener(self.app_id, func)
		return self

	# -------------------------------------
	def add_render_callback(self, func):
		# -------------------------------------
		ac.addRenderCallback(self.app_id, func)
		return self

	# -------------------------------------
	def set_color(self, rgb, a=1):
		# -------------------------------------
		self.rgb = rgb
		self.alpha = a
		ac.setFontColor(self.app_id, rgb.red, rgb.green, rgb.blue, a)
		return self

	# -------------------------------------
	def set_alpha(self, a):
		# -------------------------------------
		self.alpha = a
		self.set_color(self.rgb, a)

	# -------------------------------------
	def set_visible(self, visible=True):
		# -------------------------------------
		self.is_visible = visible
		ac.setVisible(self.app_id, int(visible))
		if self.children:
			for child in self.children:
				child.set_visible(visible)
		return self

	# -------------------------------------
	def hide(self):
		# -------------------------------------
		self.set_visible(False)
		return self

	# -------------------------------------
	def show(self):
		# -------------------------------------
		self.set_visible(True)
		return self

	# -------------------------------------
	def set_size(self, *xy):
		# -------------------------------------
		self.size = XY(*xy)
		ac.setSize(self.app_id, scale(self.size.x), scale(self.size.y))
		return self

	# -------------------------------------
	def set_border(self, border):
		# -------------------------------------
		self.border = border
		ac.drawBorder(self.app_id, self.border)
		return self

	# -------------------------------------
	def set_bg_opacity(self, opacity):
		# -------------------------------------
		self.opacity = opacity
		ac.setBackgroundOpacity(self.app_id, self.opacity)
		return self

	# -------------------------------------
	def align(self, alignment):
		# -------------------------------------
		ac.setFontAlignment(self.app_id, alignment)
		return self

	# -------------------------------------
	def hide_icon(self):
		# -------------------------------------
		ac.setIconPosition(self.app_id, 0, -10000)
		return self

	# -------------------------------------
	def hide_title(self):
		# -------------------------------------
		ac.setTitlePosition(self.app_id, 0, -10000)
		return self

	# -------------------------------------
	def set_pos(self, *xy, parent=None):
		# -------------------------------------
		self.pos = XY(*xy)
		if (parent and parent.pos):
			ac.setPosition(self.app_id, scale(self.pos.x + parent.pos.x), scale(self.pos.y + parent.pos.y))
		else:
			ac.setPosition(self.app_id, scale(self.pos.x), scale(self.pos.y))
		if self.children:
			for child in self.children:
				child.set_pos(*child.pos, parent=self)
		return self

	# -------------------------------------
	def get_offset(self):
		# -------------------------------------
		return XY(self.pos.x + self.size.x, self.pos.y + self.size.y)

	# -------------------------------------
	def set_bg_image(self, file):
		# -------------------------------------
		ac.setBackgroundTexture(self.app_id, file)
		return self

	# -------------------------------------
	def set_bg_color(self, rgb):
		# -------------------------------------
		ac.setBackgroundColor(self.app_id, rgb.red, rgb.green, rgb.blue)
		# background is not updated if the opacity does not change.
		self.refresh_opacity()
		return self

	# -------------------------------------
	def set_font_size(self, size):
		# -------------------------------------
		ac.setFontSize(self.app_id, scale(size))
		if self.children:
			for child in self.children:
				child.set_font_size(size)
		return self

	# -------------------------------------
	def set_text(self, text):
		# -------------------------------------
		ac.setText(self.app_id, text)
		return self

	# -------------------------------------
	def get_text(self):
		# -------------------------------------
		return ac.getText(self.app_id)

	# -------------------------------------
	def add_child(self, *children):
		# -------------------------------------
		if not self.children:
			self.children = []
		for child in children:
			self.children.append(child)
		return self

# -------------------------------------
class Label(Control):
	# -------------------------------------
	# -------------------------------------
	def __init__(self, text=None, app_id=None):
		# -------------------------------------
		if text:
			super().__init__(ac.addLabel(app_id, text))
		else:
			super().__init__(ac.addLabel(app_id, ''))

# -------------------------------------
class SplitLabel(Label):
	# -------------------------------------
	# -------------------------------------
	def __init__(self, split=None, app_id=None):
		# -------------------------------------
		super().__init__(text=None, app_id=app_id)
		self.set_split(split).align('right').set_size(64, 40)

	# -------------------------------------
	def set_split(self, split):
		# -------------------------------------
		self.set_text('%.3f' % (split / 1000))
		return self

# -------------------------------------
class DiffLabel(Label):
	# -------------------------------------
	# -------------------------------------
	def __init__(self, diff=None, app_id=None):
		# -------------------------------------
		super().__init__(text=None, app_id=app_id)
		self.set_diff(diff).align('right')

	# -------------------------------------
	def set_diff(self, diff):
		# -------------------------------------
		t = diff / 1000

		if t < 0:
			# green - diff
			self.set_text('%.3f' % t).set_color(GREEN)
		elif t > 0:
			# red + diff
			self.set_text('+%.3f' % t).set_color(LIGHT_RED)
		else:
			# white 0 diff
			self.set_text('%.3f' % t).set_color(WHITE)

		return self

# -------------------------------------
class TimeLabel(Label):
	# -------------------------------------
	# -------------------------------------
	def __init__(self, time=None, app_id=None):
		# -------------------------------------
		super().__init__(text=time_to_string(time), app_id=app_id)

	# -------------------------------------
	def set_time(self, time):
		# -------------------------------------
		self.set_text(time_to_string(time))
		return self

# -------------------------------------
class Alert(Label):
	# -------------------------------------
	flash_speed = 0.03

	# -------------------------------------
	def __init__(self, text=None, app_id=None):
		# -------------------------------------
		super().__init__(text=text, app_id=app_id)
		self.flash = False
		self.active = False

	# -------------------------------------
	def splash(self, s):
		# -------------------------------------
		self.active = True
		self.set_text(s).set_alpha(1)
		self.show()
		return self

	# -------------------------------------
	def update_fade(self):
		# -------------------------------------
		if self.active:
			alpha = self.alpha

			if alpha >= 1 or alpha <= 0.25:
				self.flash = not self.flash

			speed = Alert.flash_speed

			alpha += -speed if self.flash else speed

			self.set_alpha(alpha)
		return self

# -------------------------------------
class Image(Label):
	# -------------------------------------
	# -------------------------------------
	def __init__(self, filename=None, app_id=None):
		# -------------------------------------
		super().__init__(text=None, app_id=app_id)
		self.set_bg_image(filename)

# -------------------------------------
class Icon(Image):
	# -------------------------------------
	# -------------------------------------
	def set_enabled(self, val):
		# -------------------------------------
		if val:
			self.set_bg_color(RED)
			self.set_bg_opacity(0.5)
		else:
			self.set_bg_color(BLACK)
			self.set_bg_opacity(0.5)

		return self

# -------------------------------------
class Texture():
	# -------------------------------------
	# -------------------------------------
	def __init__(self, filename):
		# -------------------------------------
		self.app_id = ac.newTexture(filename)

	# -------------------------------------
	def render(self, x, y, width, height):
		# -------------------------------------
		ac.glQuadTextured(x, y, width, height, self.app_id)

# -------------------------------------
class ToggleImage(Label):
	# -------------------------------------
	# -------------------------------------
	def __init__(self, filenames=None, app_id=None):
		# -------------------------------------
		super().__init__(text=None, app_id=app_id)
		self.filenames = filenames
		self.toggle()

	# -------------------------------------
	def toggle(self, current=0):
		# -------------------------------------
		self.set_bg_image(self.filenames[current])

# -------------------------------------
class ToggleableLabelAdapter(object):
	# -------------------------------------
	# -------------------------------------
	def __init__(self, label, texts, values=None):
		# -------------------------------------
		super().__init__()
		self.label = label
		self.texts = texts
		self.values = values or texts
		self._current_i = 0
		self.toggle(0)

	# -------------------------------------
	def toggle(self, current=None):
		# -------------------------------------
		if current is None:
			self._current_i += 1
			if self._current_i >= len(self.texts):
				self._current_i = 0
		else:
			assert current < len(self.texts)
			self._current_i = current
		from .utils import log

		log('toggle %s - %s - %s' % (self._current_i, self.current_value, self.current_text))
		self.set_text(self.current_text)
		return self

	# -------------------------------------
	@property
	def current_text(self):
		# -------------------------------------
		return self.texts[self._current_i]

	# -------------------------------------
	@property
	def current_value(self):
		# -------------------------------------
		return self.values[self._current_i]

	# -------------------------------------
	@current_value.setter
	def current_value(self, new_value):
		# -------------------------------------
		self._current_i = self.values.index(new_value)
		self.set_text(self.current_text)

	# -------------------------------------
	def __getattr__(self, item):
		# -------------------------------------
		return getattr(self.label, item)

# -------------------------------------
class Grid:
	# -------------------------------------
	# -------------------------------------
	def __init__(self, nrows, ncols, row_height, col_widths, cell_class=Label, font_size=None, x=0, y=0, horizontal_gap=0, vertical_gap=1, app_id=None):
		# -------------------------------------
		self.rows = []
		self.vertical_gap = vertical_gap
		self.horizontal_gap = horizontal_gap

		for row_num in range(nrows):
			cols = []
			y_offset = row_num * (row_height + vertical_gap)
			for col_num in range(ncols):
				cell = cell_class(app_id=app_id)
				if col_num == 0:
					x_offset = 0
				else:
					x_offset = sum(col_widths[:col_num]) + col_num * horizontal_gap
				cell.set_size(col_widths[col_num], row_height).set_pos(x + x_offset, y + y_offset)
				if font_size:
					cell.set_font_size(font_size)
				cols.append(cell)
			self.rows.append(cols)

	# -------------------------------------
	def hide_column(self, n):
		# -------------------------------------
		for row in self.rows:
			row[n].hide()

	# -------------------------------------
	def show_column(self, n):
		# -------------------------------------
		for row in self.rows:
			row[n].show()

	# -------------------------------------
	def hide_row(self, n):
		# -------------------------------------
		for col in self.rows[n]:
			col.hide()

	# -------------------------------------
	def show_row(self, n):
		# -------------------------------------
		for col in self.rows[n]:
			col.show()

	# -------------------------------------
	def clear_row(self, row):
		# -------------------------------------
		for cell in self.rows[row]:
			cell.set_text('')

	# -------------------------------------
	def set_texts(self, row, *texts):
		# -------------------------------------
		for cell, text in zip(self.rows[row], texts):
			cell.set_text(text)

	# -------------------------------------
	@property
	def width(self):
		# -------------------------------------
		return sum(c.size.x + self.horizontal_gap for c in self.rows[0] if c.is_visible)

	# -------------------------------------
	@property
	def height(self):
		# -------------------------------------
		return sum(r[0].size.y + self.vertical_gap for r in self.rows if r[0].is_visible)

	# -------------------------------------
	@property
	def max_height(self):
		# -------------------------------------
		return sum(r[0].size.y + self.vertical_gap for r in self.rows)

# -------------------------------------
def time_to_string(t, include_ms=True):
	# -------------------------------------
	try:
		hours, x = divmod(int(t), 3600000)
		mins, x = divmod(x, 60000)
		secs, ms = divmod(x, 1000)
		if not include_ms:
			return '%d:%02d' % (mins, secs)
		return '%d.%03d' % (secs, ms) if mins == 0 else '%d:%02d.%03d' % (mins, secs, ms)
	except Exception:
		return '--:--.---'
