# !/usr/bin/env python
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Title       : applaps.py
# Author      : aDd1kTeD2Ka0s (add1kted2ka0s@gmail.com)
# Date        : 20-Jun-2014
# Description : Ka0s Leaderboard - AC application - Laps
#             : Original code by Rombik
# -----------------------------------------------------------------------------

# -------------------------------------
# Imports
# -------------------------------------

import ac
import time

from .appdata import app_data
from .config import *
from ..lib.utils import log
from ..lib.widgets import Control, Grid, Label, time_to_string

# -------------------------------------
class AppLaps(Control):
	# -------------------------------------

	NAME = 'Ka0s Laps'

	# -------------------------------------
	def __init__(self):
		# -------------------------------------
		self.app_id = ac.newApp(self.NAME)
		super().__init__(self.app_id)

		self.last_timestamp = 0

		self.mode_selection_dict = dict()
		self.mode_selection_list = list()
		self.mode_selection_id_min = None
		self.mode_selection_id_max = None
		self.mode_selection_id_current = None

		self.column_layout = COLUMN_LAYOUT_TEMPLATES_LAPS[0]['layout']

	# -------------------------------------
	def on_startup(self, ac_version):
		# -------------------------------------
		# Define window elements parameters
		self.calculate_window_parameters()

		# Initialize grid
		self.grid = Grid(self.grid_num_rows, self.total_num_grid_columns, ROW_HEIGHT, [1] * self.total_num_grid_columns, font_size=FONT_SIZE, x=WINDOW_BORDER, y=self.grid_top_offset, app_id=self.app_id)

		# Mode selection text
		self.mode_selection_label = Label(text='Layout', app_id=self.app_id)
		self.mode_selection_label.set_font_size(FONT_SIZE).align('center')
		self.mode_selection_label.set_color(COLOR_FOREGROUND_MODE_SELECTION_TEXT).set_bg_color(COLOR_BACKGROUND_MODE_SELECTION_TEXT).set_bg_opacity(OPACITY_BACKGROUND_MODE_SELECTION_TEXT)

		# Mode selection selector left
		self.mode_selection_left_label = Label(text='<', app_id=self.app_id)
		self.mode_selection_left_label.set_font_size(FONT_SIZE).align('center')
		self.mode_selection_left_label.set_color(COLOR_FOREGROUND_MODE_SELECTION_SELECTOR).set_bg_color(COLOR_BACKGROUND_MODE_SELECTION_SELECTOR).set_bg_opacity(OPACITY_BACKGROUND_MODE_SELECTION_SELECTOR)
		self.mode_selection_left_label.add_click_listener(mode_selection_left_label_on_clicked)

		# Mode selection selector right
		self.mode_selection_right_label = Label(text='>', app_id=self.app_id)
		self.mode_selection_right_label.set_font_size(FONT_SIZE).align('center')
		self.mode_selection_right_label.set_color(COLOR_FOREGROUND_MODE_SELECTION_SELECTOR).set_bg_color(COLOR_BACKGROUND_MODE_SELECTION_SELECTOR).set_bg_opacity(OPACITY_BACKGROUND_MODE_SELECTION_SELECTOR)
		self.mode_selection_right_label.add_click_listener(mode_selection_right_label_on_clicked)

		# Initialize mode selection
		self.mode_selection_id_current = 0  # Initially set to mode #0 - i.e. Default
		self.mode_selection_label.set_text(ac.getDriverName(self.mode_selection_id_current))

		# Update window
		self.update_window()

		return self.NAME

	# -------------------------------------
	def on_update(self, dt):
		# -------------------------------------
		# Refresh display only every UPDATE_FREQUENCY milliseconds
		current_timestamp = time.time()
		if (current_timestamp - self.last_timestamp) < UPDATE_FREQUENCY:
			return
		self.last_timestamp = current_timestamp

		best_worst = app_data.cars.calc_best_worst_laps()

		self.refresh_opacity()

		for grid_row_id, cols in enumerate(self.grid.rows[self.grid_header_rows:]):
			car_id = self.mode_selection_id_current
			car_state = app_data.cars.car_states[car_id]
			lap_num = car_state.completed_laps - grid_row_id + 1

			if grid_row_id >= car_state.completed_laps + 1:
				self.grid.clear_row(grid_row_id + self.grid_header_rows)
				continue

			if grid_row_id % 2 == 0:
				color_foreground = COLOR_FOREGROUND_ROW_1
				color_background = COLOR_BACKGROUND_ROW_1
				opacity_background = OPACITY_BACKGROUND_ROW_1
			else:
				color_foreground = COLOR_FOREGROUND_ROW_2
				color_background = COLOR_BACKGROUND_ROW_2
				opacity_background = OPACITY_BACKGROUND_ROW_2

			# Set label texts
			# --- COL_LAP_NUM ---
			if 'COL_LAP_NUM' in self.grid_column_positions:
				formatted_string = '%-2d' % (lap_num)
				cols[self.grid_column_positions['COL_LAP_NUM']].set_text(formatted_string).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
			# --- COL_SPLIT ---
			for split_index in range(SPLIT_COUNT):
				if 'COL_SPLIT_%d' % (split_index + 1) in self.grid_column_positions:
					formatted_string = '%.3f' % (car_state.splits[lap_num][split_index] / 1000)
					cols[self.grid_column_positions['COL_SPLIT_%d' % (split_index + 1)]].set_text(formatted_string).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
					if ENABLE_BEST_WORST_COLOR:
						if best_worst[car_id]['splits']['best'][split_index]['lap_number'] == lap_num:
							self.update_column_best_worst(cols[self.grid_column_positions['COL_SPLIT_%d' % (split_index + 1)]], COLOR_BEST)
						elif best_worst[car_id]['splits']['worst'][split_index]['lap_number'] == lap_num:
							self.update_column_best_worst(cols[self.grid_column_positions['COL_SPLIT_%d' % (split_index + 1)]], COLOR_WORST)
			# --- COL_LAP_TIME ---
			if 'COL_LAP_TIME' in self.grid_column_positions:
				cols[self.grid_column_positions['COL_LAP_TIME']].set_text(time_to_string(car_state.laps[lap_num])).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
				if ENABLE_BEST_WORST_COLOR:
					if best_worst[car_id]['lap_time']['best']['lap_number'] == lap_num:
						self.update_column_best_worst(cols[self.grid_column_positions['COL_LAP_TIME']], COLOR_BEST)
					elif best_worst[car_id]['lap_time']['worst']['lap_number'] == lap_num:
						self.update_column_best_worst(cols[self.grid_column_positions['COL_LAP_TIME']], COLOR_WORST)

	# -------------------------------------
	def trim_column(self, col_name, value, extra_time=0):
		# -------------------------------------
		return value[:COL_SETTINGS_LAPS[col_name]['width'] - extra_time]

	# -------------------------------------
	def update_column_best_worst(self, column, color_dict):
		# -------------------------------------
		if COLOR_FOREGROUND_BEST:
			column.set_color(color_dict['foreground'])
		if COLOR_BACKGROUND_BEST:
			column.set_bg_color(color_dict['background'])
		if OPACITY_BACKGROUND_BEST:
			column.set_bg_opacity(color_dict['opacity'])

	# -------------------------------------
	def calculate_window_parameters(self):
		# -------------------------------------
		self.window_max_width = WINDOW_BORDER + WINDOW_PIN_WIDTH
		self.grid_column_labels = tuple()
		self.grid_column_widths = tuple()
		self.grid_column_aligns = tuple()
		self.grid_column_x_position = tuple()
		self.grid_column_positions = dict()

		column_counter = 0
		x_position = WINDOW_BORDER
		for entry_id in self.column_layout:
			entry = COL_SETTINGS_LAPS[entry_id]
			if entry_id == 'COL_SPLIT':
				for i in range(SPLIT_COUNT):
					self.grid_column_labels += (entry['label'] + str(i + 1),)
					self.grid_column_widths += (round(LETTER_WIDTH * entry['width']),)
					self.grid_column_aligns += (entry['align'],)
					self.grid_column_x_position += (x_position,)
					self.grid_column_positions[entry_id + '_' + str(i + 1)] = column_counter
					self.window_max_width += (LETTER_WIDTH * entry['width'])
					x_position += round(LETTER_WIDTH * entry['width'])
					column_counter += 1
			else:
				self.grid_column_labels += (entry['label'],)
				self.grid_column_widths += (round(LETTER_WIDTH * entry['width']),)
				self.grid_column_aligns += (entry['align'],)
				self.grid_column_x_position += (x_position,)
				self.grid_column_positions[entry_id] = column_counter
				self.window_max_width += (LETTER_WIDTH * entry['width'])
				x_position += round(LETTER_WIDTH * entry['width'])
				column_counter += 1

		self.grid_header_rows = 1
		self.grid_num_rows = self.grid_header_rows + NUMBER_OF_ROWS_APP_LAPS
		self.mode_selection_height = ROW_HEIGHT + 3
		self.mode_selection_label_width = round(LETTER_WIDTH * 3)
		self.grid_top_offset = WINDOW_BORDER + self.mode_selection_height + WINDOW_BORDER
		self.window_max_height = self.grid_top_offset + (ROW_HEIGHT * self.grid_num_rows) + WINDOW_BORDER

		self.total_num_grid_columns = len(COL_SETTINGS_LAPS)
		if 'COL_SPLIT' in COL_SETTINGS_LAPS:
			self.total_num_grid_columns += SPLIT_COUNT - 1

		self.active_num_grid_columns = len(self.column_layout)
		if 'COL_SPLIT' in self.column_layout:
			self.active_num_grid_columns += SPLIT_COUNT - 1

	# -------------------------------------
	def update_window(self):
		# -------------------------------------
		# Define window element parameters
		self.calculate_window_parameters()

		# Set window size
		self.set_size(self.window_max_width, self.window_max_height).hide_icon().hide_title().set_bg_opacity(WINDOW_OPACITY).set_border(0)

		# Show/hide columns
		for i in range(self.active_num_grid_columns):
			self.grid.show_column(i)
		for i in range(self.active_num_grid_columns, self.total_num_grid_columns):
			self.grid.hide_column(i)

		# Set header row
		for row in self.grid.rows[:self.grid_header_rows]:
			for column_id, column in enumerate(range(self.active_num_grid_columns)):
				row[column_id].set_bg_color(COLOR_BACKGROUND_HEADER).set_bg_opacity(OPACITY_BACKGROUND_HEADER)
				row[column_id].set_text(self.grid_column_labels[column_id])

		# Update every column
		y_position = self.grid_top_offset
		for row_id, row in enumerate(self.grid.rows):
			for column_id, column in enumerate(range(self.active_num_grid_columns)):
				row[column_id].set_size(self.grid_column_widths[column_id], ROW_HEIGHT)
				row[column_id].set_pos(self.grid_column_x_position[column_id], y_position)
				row[column_id].align(self.grid_column_aligns[column_id])
			y_position += ROW_HEIGHT

		# Update mode selection
		self.mode_selection_label.set_pos(WINDOW_BORDER + self.mode_selection_label_width, WINDOW_BORDER)
		self.mode_selection_label.set_size(sum(self.grid_column_widths) - self.mode_selection_label_width * 2, self.mode_selection_height)
		self.mode_selection_left_label.set_pos(WINDOW_BORDER, WINDOW_BORDER)
		self.mode_selection_left_label.set_size(self.mode_selection_label_width, self.mode_selection_height)
		self.mode_selection_right_label.set_pos(WINDOW_BORDER + sum(self.grid_column_widths) - self.mode_selection_label_width, WINDOW_BORDER)
		self.mode_selection_right_label.set_size(self.mode_selection_label_width, self.mode_selection_height)

	# -------------------------------------
	def init_mode_selection_dictionary(self):
		# -------------------------------------
		for car_state in app_data.cars.car_states:
			self.mode_selection_dict[car_state.car_id] = {'car_name': car_state.car_name, 'driver_name': car_state.driver_name}
			self.mode_selection_list.append(car_state.car_id)
			if self.mode_selection_id_min is None or car_state.car_id < self.mode_selection_id_min:
				self.mode_selection_id_min = car_state.car_id
			if self.mode_selection_id_max is None or car_state.car_id > self.mode_selection_id_max:
				self.mode_selection_id_max = car_state.car_id
		self.mode_selection_list = sorted(self.mode_selection_list)

	# -------------------------------------
	def mode_selection_left_label_on_clicked(self):
		# -------------------------------------
		# Init mode selection dictionary if it doesnt exist
		if self.mode_selection_id_min is None:
			self.init_mode_selection_dictionary()

		# Get selected mode
		if self.mode_selection_id_current is None:
			selected_mode = 0
		else:
			index = self.mode_selection_list.index(self.mode_selection_id_current)
			if index == self.mode_selection_id_min:
				selected_mode = self.mode_selection_id_max
			else:
				selected_mode = self.mode_selection_list[index - 1]

		# Set selected mode
		self.mode_selection_id_current = selected_mode
		self.mode_selection_label.set_text(self.mode_selection_dict[selected_mode]['driver_name'])

		# Update window
		self.update_window()

	# -------------------------------------
	def mode_selection_right_label_on_clicked(self):
		# -------------------------------------
		# Init mode selection dictionary if it doesnt exist
		if self.mode_selection_id_min is None:
			self.init_mode_selection_dictionary()

		# Get selected mode
		if self.mode_selection_id_current is None:
			selected_mode = 0
		else:
			index = self.mode_selection_list.index(self.mode_selection_id_current)
			if index == self.mode_selection_id_max:
				selected_mode = self.mode_selection_id_min
			else:
				selected_mode = self.mode_selection_list[index + 1]

		# Set selected mode
		self.mode_selection_id_current = selected_mode
		self.mode_selection_label.set_text(self.mode_selection_dict[selected_mode]['driver_name'])

		# Update window
		self.update_window()

# -------------------------------------
def mode_selection_left_label_on_clicked(*args):
	# -------------------------------------
	app_laps.mode_selection_left_label_on_clicked()

# -------------------------------------
def mode_selection_right_label_on_clicked(*args):
	# -------------------------------------
	app_laps.mode_selection_right_label_on_clicked()

# -------------------------------------
# Main
# -------------------------------------
app_laps = AppLaps()
