# !/usr/bin/env python
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Title       : appleaderboard.py
# Author      : aDd1kTeD2Ka0s (add1kted2ka0s@gmail.com)
# Date        : 20-Jun-2014
# Description : Ka0s Leaderboard - AC application - Leaderboard
#             : Original code by Rombik
# -----------------------------------------------------------------------------

# -------------------------------------
# Imports
# -------------------------------------

import ac
import time

from .appdata import app_data
from .config import *
from ..lib.utils import log
from ..lib.widgets import Control, Grid, Label, time_to_string

# -------------------------------------
class AppLeaderboard(Control):
	# -------------------------------------

	NAME = 'Ka0s Leaderboard'

	# -------------------------------------
	def __init__(self):
		# -------------------------------------
		self.app_id = ac.newApp(self.NAME)
		super().__init__(self.app_id)

		self.last_timestamp = 0

		self.mode_selection_dict = dict()
		self.mode_selection_list = list()
		self.mode_selection_id_min = None
		self.mode_selection_id_max = None
		self.mode_selection_id_current = None

		self.column_layout = COLUMN_LAYOUT_TEMPLATES_LEADERBOARD[0]['layout']

	# -------------------------------------
	def on_startup(self, ac_version):
		# -------------------------------------
		# Define window elements parameters
		self.calculate_window_parameters()

		# Initialize grid
		self.grid = Grid(self.grid_num_rows, self.total_num_grid_columns, ROW_HEIGHT, [1] * self.total_num_grid_columns, font_size=FONT_SIZE, x=WINDOW_BORDER, y=self.grid_top_offset, app_id=self.app_id)

		# Mode selection text
		self.mode_selection_label = Label(text='Layout', app_id=self.app_id)
		self.mode_selection_label.set_font_size(FONT_SIZE).align('left')
		self.mode_selection_label.set_color(COLOR_FOREGROUND_MODE_SELECTION_TEXT).set_bg_color(COLOR_BACKGROUND_MODE_SELECTION_TEXT).set_bg_opacity(OPACITY_BACKGROUND_MODE_SELECTION_TEXT)

		# Mode selection selector left
		self.mode_selection_left_label = Label(text='<', app_id=self.app_id)
		self.mode_selection_left_label.set_font_size(FONT_SIZE).align('center')
		self.mode_selection_left_label.set_color(COLOR_FOREGROUND_MODE_SELECTION_SELECTOR).set_bg_color(COLOR_BACKGROUND_MODE_SELECTION_SELECTOR).set_bg_opacity(OPACITY_BACKGROUND_MODE_SELECTION_SELECTOR)
		self.mode_selection_left_label.add_click_listener(mode_selection_left_label_on_clicked)

		# Mode selection selector right
		self.mode_selection_right_label = Label(text='>', app_id=self.app_id)
		self.mode_selection_right_label.set_font_size(FONT_SIZE).align('center')
		self.mode_selection_right_label.set_color(COLOR_FOREGROUND_MODE_SELECTION_SELECTOR).set_bg_color(COLOR_BACKGROUND_MODE_SELECTION_SELECTOR).set_bg_opacity(OPACITY_BACKGROUND_MODE_SELECTION_SELECTOR)
		self.mode_selection_right_label.add_click_listener(mode_selection_right_label_on_clicked)

		# Initialize mode selection
		self.mode_selection_id_current = 0  # Initially set to mode #0 - i.e. Default
		self.mode_selection_label.set_text(COLUMN_LAYOUT_TEMPLATES_LEADERBOARD[self.mode_selection_id_current]['name'])

		# Update window
		self.update_window()

		return self.NAME

	# -------------------------------------
	def on_update(self, dt):
		# -------------------------------------
		# Refresh display only every UPDATE_FREQUENCY milliseconds
		current_timestamp = time.time()
		if (current_timestamp - self.last_timestamp) < UPDATE_FREQUENCY:
			return
		self.last_timestamp = current_timestamp

		visible_cars_deltas = app_data.cars.calc_deltas()
		best_worst = app_data.cars.calc_best_worst_leaderboard()

		self.refresh_opacity()

		for grid_row_id, cols in enumerate(self.grid.rows[self.grid_header_rows:]):

			if grid_row_id >= len(app_data.cars.visible_grid):
				self.grid.clear_row(grid_row_id + self.grid_header_rows)
				continue

			(grid_pos, car_id) = app_data.cars.visible_grid[grid_row_id]
			car_state = app_data.cars.car_states[car_id]
			car_delta = visible_cars_deltas[grid_row_id]

			if grid_row_id % 2 == 0:
				color_foreground = COLOR_FOREGROUND_ROW_1
				color_background = COLOR_BACKGROUND_ROW_1
				opacity_background = OPACITY_BACKGROUND_ROW_1
			else:
				color_foreground = COLOR_FOREGROUND_ROW_2
				color_background = COLOR_BACKGROUND_ROW_2
				opacity_background = OPACITY_BACKGROUND_ROW_2

			if car_id == 0 and ENABLE_SELF_COLOR:
				if COLOR_FOREGROUND_SELF:
					color_foreground = COLOR_FOREGROUND_SELF
				if COLOR_BACKGROUND_SELF:
					color_background = COLOR_BACKGROUND_SELF
				if OPACITY_BACKGROUND_SELF:
					opacity_background = OPACITY_BACKGROUND_SELF

			# Set label texts
			# --- COL_POSITION ---
			if 'COL_POSITION' in self.grid_column_positions:
				formatted_string = '%-2d' % (grid_pos + 1)
				cols[self.grid_column_positions['COL_POSITION']].set_text(formatted_string).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
			# --- COL_NAME ---
			if 'COL_NAME' in self.grid_column_positions:
				trimmed_column = self.trim_column('COL_NAME', car_state.driver_name, 2)
				cols[self.grid_column_positions['COL_NAME']].set_text(trimmed_column).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
			# --- COL_CAR ---
			if 'COL_CAR' in self.grid_column_positions:
				trimmed_column = self.trim_column('COL_CAR', car_state.car_name)
				cols[self.grid_column_positions['COL_CAR']].set_text(trimmed_column).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
			# --- COL_LAPS ---
			if 'COL_LAPS' in self.grid_column_positions:
				formatted_string = '%d/%d' % (car_state.completed_laps, car_state.number_of_laps)
				cols[self.grid_column_positions['COL_LAPS']].set_text(formatted_string).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
			# --- COL_COMPLETION_LAP ---
			if 'COL_COMPLETION_LAP' in self.grid_column_positions:
				formatted_string = '%.2f%%' % (car_state.completion_lap)
				cols[self.grid_column_positions['COL_COMPLETION_LAP']].set_text(formatted_string).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
			# --- COL_COMPLETION_RACE ---
			if 'COL_COMPLETION_RACE' in self.grid_column_positions:
				formatted_string = '%.2f%%' % (car_state.completion_race)
				cols[self.grid_column_positions['COL_COMPLETION_RACE']].set_text(formatted_string).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
			# --- COL_DISTANCE_LAP ---
			if 'COL_DISTANCE_LAP' in self.grid_column_positions:
				if SPEED_UNIT == SPEED_UNIT_KMH:
					formatted_string = '%.3f %s' % (car_state.distance_lap_km, SPEED_UNIT['distance_suffix_large'])
				elif SPEED_UNIT == SPEED_UNIT_MPH:
					formatted_string = '%.2f %s' % (car_state.distance_lap_mi, SPEED_UNIT['distance_suffix_large'])
				cols[self.grid_column_positions['COL_DISTANCE_LAP']].set_text(formatted_string).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
			# --- COL_DISTANCE_RACE ---
			if 'COL_DISTANCE_RACE' in self.grid_column_positions:
				if SPEED_UNIT == SPEED_UNIT_KMH:
					formatted_string = '%.3f %s' % (car_state.distance_race_km, SPEED_UNIT['distance_suffix_large'])
				elif SPEED_UNIT == SPEED_UNIT_MPH:
					formatted_string = '%.2f %s' % (car_state.distance_race_mi, SPEED_UNIT['distance_suffix_large'])
				cols[self.grid_column_positions['COL_DISTANCE_RACE']].set_text(formatted_string).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
			# --- COL_DIFF_TIME_LEAD ---
			if 'COL_DIFF_TIME_LEAD' in self.grid_column_positions:
				sign_time_lead = '+' if car_delta.delta_time_lead is not None and car_delta.delta_time_lead >= 0 else '-'
				delta_time_lead = '%s%.3fs' % (sign_time_lead, abs(car_delta.delta_time_lead)) if car_delta.delta_time_lead is not None else '+0.000'
				cols[self.grid_column_positions['COL_DIFF_TIME_LEAD']].set_text(delta_time_lead).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
			# --- COL_DIFF_TIME_PREV ---
			if 'COL_DIFF_TIME_PREV' in self.grid_column_positions:
				sign_time_prev = '+' if car_delta.delta_time_prev is not None and car_delta.delta_time_prev >= 0 else '-'
				delta_time_prev = '%s%.3fs' % (sign_time_prev, abs(car_delta.delta_time_prev)) if car_delta.delta_time_prev is not None else '+0.000'
				cols[self.grid_column_positions['COL_DIFF_TIME_PREV']].set_text(delta_time_prev).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
			# --- COL_DIFF_TIME_YOU ---
			if 'COL_DIFF_TIME_YOU' in self.grid_column_positions:
				sign_time_you = '+' if car_delta.delta_time_you is not None and car_delta.delta_time_you >= 0 else '-'
				delta_time_you = '%s%.3fs' % (sign_time_you, abs(car_delta.delta_time_you)) if car_delta.delta_time_you is not None else '+0.000'
				cols[self.grid_column_positions['COL_DIFF_TIME_YOU']].set_text(delta_time_you).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
			# --- COL_DIFF_DISTANCE_LEAD ---
			if 'COL_DIFF_DISTANCE_LEAD' in self.grid_column_positions:
				sign_distance_lead = '+' if car_delta.delta_distance_lead_km is not None and car_delta.delta_distance_lead_km >= 0 else '-'
				if SPEED_UNIT == SPEED_UNIT_KMH:
					delta_distance_lead = '%s%.3f %s' % (sign_distance_lead, abs(car_delta.delta_distance_lead_km), SPEED_UNIT['distance_suffix_large']) if car_delta.delta_distance_lead_km is not None else '+0.000'
				elif SPEED_UNIT == SPEED_UNIT_MPH:
					delta_distance_lead = '%s%.3f %s' % (sign_distance_lead, abs(car_delta.delta_distance_lead_mi), SPEED_UNIT['distance_suffix_large']) if car_delta.delta_distance_lead_mi is not None else '+0.000'
				else:
					delta_distance_lead = '+0.000'
				cols[self.grid_column_positions['COL_DIFF_DISTANCE_LEAD']].set_text(delta_distance_lead).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
			# --- COL_DIFF_DISTANCE_PREV ---
			if 'COL_DIFF_DISTANCE_PREV' in self.grid_column_positions:
				sign_distance_prev = '+' if car_delta.delta_distance_prev_km is not None and car_delta.delta_distance_prev_km >= 0 else '-'
				if SPEED_UNIT == SPEED_UNIT_KMH:
					delta_distance_prev = '%s%.3f %s' % (sign_distance_prev, abs(car_delta.delta_distance_prev_km), SPEED_UNIT['distance_suffix_large']) if car_delta.delta_distance_prev_km is not None else '+0.000'
				elif SPEED_UNIT == SPEED_UNIT_MPH:
					delta_distance_prev = '%s%.3f %s' % (sign_distance_prev, abs(car_delta.delta_distance_prev_mi), SPEED_UNIT['distance_suffix_large']) if car_delta.delta_distance_prev_mi is not None else '+0.000'
				else:
					delta_distance_prev = '+0.000'
				cols[self.grid_column_positions['COL_DIFF_DISTANCE_PREV']].set_text(delta_distance_prev).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
			# --- COL_DIFF_DISTANCE_YOU ---
			if 'COL_DIFF_DISTANCE_YOU' in self.grid_column_positions:
				sign_distance_you = '+' if car_delta.delta_distance_you_km is not None and car_delta.delta_distance_you_km >= 0 else '-'
				if SPEED_UNIT == SPEED_UNIT_KMH:
					delta_distance_you = '%s%.3f %s' % (sign_distance_you, abs(car_delta.delta_distance_you_km), SPEED_UNIT['distance_suffix_large']) if car_delta.delta_distance_you_km is not None else '+0.000'
				elif SPEED_UNIT == SPEED_UNIT_MPH:
					delta_distance_you = '%s%.3f %s' % (sign_distance_you, abs(car_delta.delta_distance_you_mi), SPEED_UNIT['distance_suffix_large']) if car_delta.delta_distance_you_mi is not None else '+0.000'
				else:
					delta_distance_you = '+0.000'
				cols[self.grid_column_positions['COL_DIFF_DISTANCE_YOU']].set_text(delta_distance_you).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
			# --- COL_SPLIT ---
			for split_index in range(SPLIT_COUNT):
				if 'COL_SPLIT_%d' % (split_index + 1) in self.grid_column_positions:
					formatted_string = '%.3f' % (car_state.live_splits[split_index] / 1000)
					cols[self.grid_column_positions['COL_SPLIT_%d' % (split_index + 1)]].set_text(formatted_string).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
					if ENABLE_BEST_WORST_COLOR:
						if best_worst['splits']['best'][split_index]['car_id'] == car_id:
							self.update_column_best_worst(cols[self.grid_column_positions['COL_SPLIT_%d' % (split_index + 1)]], COLOR_BEST)
						elif best_worst['splits']['worst'][split_index]['car_id'] == car_id:
							self.update_column_best_worst(cols[self.grid_column_positions['COL_SPLIT_%d' % (split_index + 1)]], COLOR_WORST)
			# --- COL_CURR_LAP ---
			if 'COL_CURR_LAP' in self.grid_column_positions:
				cols[self.grid_column_positions['COL_CURR_LAP']].set_text(time_to_string(car_state.current_lap)).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
			# --- COL_LAST_LAP ---
			if 'COL_LAST_LAP' in self.grid_column_positions:
				cols[self.grid_column_positions['COL_LAST_LAP']].set_text(time_to_string(car_state.last_lap)).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
				if ENABLE_BEST_WORST_COLOR:
					if best_worst['last_lap']['best']['car_id'] == car_id:
						self.update_column_best_worst(cols[self.grid_column_positions['COL_LAST_LAP']], COLOR_BEST)
					elif best_worst['last_lap']['worst']['car_id'] == car_id:
						self.update_column_best_worst(cols[self.grid_column_positions['COL_LAST_LAP']], COLOR_WORST)
			# --- COL_BEST_LAP ---
			if 'COL_BEST_LAP' in self.grid_column_positions:
				cols[self.grid_column_positions['COL_BEST_LAP']].set_text(time_to_string(car_state.best_lap)).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
				if ENABLE_BEST_WORST_COLOR:
					if best_worst['best_lap']['best']['car_id'] == car_id:
						self.update_column_best_worst(cols[self.grid_column_positions['COL_BEST_LAP']], COLOR_BEST)
					elif best_worst['best_lap']['worst']['car_id'] == car_id:
						self.update_column_best_worst(cols[self.grid_column_positions['COL_BEST_LAP']], COLOR_WORST)
			# --- COL_SPEED_CURRENT ---
			if 'COL_SPEED_CURRENT' in self.grid_column_positions:
				if SPEED_UNIT == SPEED_UNIT_KMH:
					formatted_string = '%d %s' % (car_state.speed_kmh_current, SPEED_UNIT['speed_suffix'])
				elif SPEED_UNIT == SPEED_UNIT_MPH:
					formatted_string = '%d %s' % (car_state.speed_mph_current, SPEED_UNIT['speed_suffix'])
				cols[self.grid_column_positions['COL_SPEED_CURRENT']].set_text(formatted_string).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
			# --- COL_SPEED_MAX ---
			if 'COL_SPEED_MAX' in self.grid_column_positions:
				if SPEED_UNIT == SPEED_UNIT_KMH:
					formatted_string = '%.2f %s' % (car_state.speed_kmh_max, SPEED_UNIT['speed_suffix'])
				elif SPEED_UNIT == SPEED_UNIT_MPH:
					formatted_string = '%.2f %s' % (car_state.speed_mph_max, SPEED_UNIT['speed_suffix'])
				cols[self.grid_column_positions['COL_SPEED_MAX']].set_text(formatted_string).set_color(color_foreground).set_bg_color(color_background).set_bg_opacity(opacity_background)
				if ENABLE_BEST_WORST_COLOR:
					if best_worst['speed_max']['best']['car_id'] == car_id:
						self.update_column_best_worst(cols[self.grid_column_positions['COL_SPEED_MAX']], COLOR_BEST)
					elif best_worst['speed_max']['worst']['car_id'] == car_id:
						self.update_column_best_worst(cols[self.grid_column_positions['COL_SPEED_MAX']], COLOR_WORST)

	# -------------------------------------
	def trim_column(self, col_name, value, extra_time=0):  # This wont work for COL_SPLIT!
		# -------------------------------------
		return value[:COL_SETTINGS_LEADERBOARD[col_name]['width'] - extra_time]

	# -------------------------------------
	def update_column_best_worst(self, column, color_dict):
		# -------------------------------------
		if COLOR_FOREGROUND_BEST:
			column.set_color(color_dict['foreground'])
		if COLOR_BACKGROUND_BEST:
			column.set_bg_color(color_dict['background'])
		if OPACITY_BACKGROUND_BEST:
			column.set_bg_opacity(color_dict['opacity'])

	# -------------------------------------
	def calculate_window_parameters(self):
		# -------------------------------------
		self.window_max_width = WINDOW_BORDER + WINDOW_PIN_WIDTH
		self.grid_column_labels = tuple()
		self.grid_column_widths = tuple()
		self.grid_column_aligns = tuple()
		self.grid_column_x_position = tuple()
		self.grid_column_positions = dict()

		column_counter = 0
		x_position = WINDOW_BORDER
		for entry_id in self.column_layout:
			entry = COL_SETTINGS_LEADERBOARD[entry_id]
			if entry_id == 'COL_SPLIT':
				for i in range(SPLIT_COUNT):
					self.grid_column_labels += (entry['label'] + str(i + 1),)
					self.grid_column_widths += (round(LETTER_WIDTH * entry['width']),)
					self.grid_column_aligns += (entry['align'],)
					self.grid_column_x_position += (x_position,)
					self.grid_column_positions[entry_id + '_' + str(i + 1)] = column_counter
					self.window_max_width += (LETTER_WIDTH * entry['width'])
					x_position += round(LETTER_WIDTH * entry['width'])
					column_counter += 1
			else:
				self.grid_column_labels += (entry['label'],)
				self.grid_column_widths += (round(LETTER_WIDTH * entry['width']),)
				self.grid_column_aligns += (entry['align'],)
				self.grid_column_x_position += (x_position,)
				self.grid_column_positions[entry_id] = column_counter
				self.window_max_width += (LETTER_WIDTH * entry['width'])
				x_position += round(LETTER_WIDTH * entry['width'])
				column_counter += 1

		self.grid_header_rows = 1
		self.grid_num_rows = self.grid_header_rows + NUMBER_OF_ROWS_APP_LEADERBOARD
		self.mode_selection_height = ROW_HEIGHT + 3
		self.mode_selection_label_width = round(LETTER_WIDTH * 3)
		self.grid_top_offset = WINDOW_BORDER + self.mode_selection_height + WINDOW_BORDER
		self.window_max_height = self.grid_top_offset + (ROW_HEIGHT * self.grid_num_rows) + WINDOW_BORDER

		self.total_num_grid_columns = len(COL_SETTINGS_LEADERBOARD)
		if 'COL_SPLIT' in COL_SETTINGS_LEADERBOARD:
			self.total_num_grid_columns += SPLIT_COUNT - 1

		self.active_num_grid_columns = len(self.column_layout)
		if 'COL_SPLIT' in self.column_layout:
			self.active_num_grid_columns += SPLIT_COUNT - 1

	# -------------------------------------
	def update_window(self):
		# -------------------------------------
		# Define window element parameters
		self.calculate_window_parameters()

		# Set window size
		self.set_size(self.window_max_width, self.window_max_height).hide_icon().hide_title().set_bg_opacity(WINDOW_OPACITY).set_border(0)

		# Show/hide columns
		for i in range(self.active_num_grid_columns):
			self.grid.show_column(i)
		for i in range(self.active_num_grid_columns, self.total_num_grid_columns):
			self.grid.hide_column(i)

		# Set header row
		for row in self.grid.rows[:self.grid_header_rows]:
			for column_id, column in enumerate(range(self.active_num_grid_columns)):
				row[column_id].set_bg_color(COLOR_BACKGROUND_HEADER).set_bg_opacity(OPACITY_BACKGROUND_HEADER)
				row[column_id].set_text(self.grid_column_labels[column_id])

		# Update every column
		y_position = self.grid_top_offset
		for row_id, row in enumerate(self.grid.rows):
			for column_id, column in enumerate(range(self.active_num_grid_columns)):
				row[column_id].set_size(self.grid_column_widths[column_id], ROW_HEIGHT)
				row[column_id].set_pos(self.grid_column_x_position[column_id], y_position)
				row[column_id].align(self.grid_column_aligns[column_id])
			y_position += ROW_HEIGHT

		# Update mode selection
		"""
		self.mode_selection_label.set_pos(WINDOW_BORDER + self.mode_selection_label_width, WINDOW_BORDER)
		self.mode_selection_label.set_size(sum(self.grid_column_widths) - self.mode_selection_label_width * 2, self.mode_selection_height)
		self.mode_selection_left_label.set_pos(WINDOW_BORDER, WINDOW_BORDER)
		self.mode_selection_left_label.set_size(self.mode_selection_label_width, self.mode_selection_height)
		self.mode_selection_right_label.set_pos(WINDOW_BORDER + sum(self.grid_column_widths) - self.mode_selection_label_width, WINDOW_BORDER)
		self.mode_selection_right_label.set_size(self.mode_selection_label_width, self.mode_selection_height)
		"""
		vertical_buffer = 10
		self.mode_selection_label.set_pos(WINDOW_BORDER + (self.mode_selection_label_width * 2) + vertical_buffer, WINDOW_BORDER)
		self.mode_selection_label.set_size(sum(self.grid_column_widths) - (self.mode_selection_label_width * 2) - vertical_buffer, self.mode_selection_height)
		self.mode_selection_left_label.set_pos(WINDOW_BORDER, WINDOW_BORDER)
		self.mode_selection_left_label.set_size(self.mode_selection_label_width, self.mode_selection_height)
		self.mode_selection_right_label.set_pos(WINDOW_BORDER + self.mode_selection_label_width, WINDOW_BORDER)
		self.mode_selection_right_label.set_size(self.mode_selection_label_width, self.mode_selection_height)

	# -------------------------------------
	def init_mode_selection_dictionary(self):
		# -------------------------------------
		for k, v in COLUMN_LAYOUT_TEMPLATES_LEADERBOARD.items():
			self.mode_selection_dict[k] = {'name': v['name'], 'layout': v['layout']}
			self.mode_selection_list.append(k)
			if self.mode_selection_id_min is None or k < self.mode_selection_id_min:
				self.mode_selection_id_min = k
			if self.mode_selection_id_max is None or k > self.mode_selection_id_max:
				self.mode_selection_id_max = k
		self.mode_selection_list = sorted(self.mode_selection_list)

	# -------------------------------------
	def mode_selection_left_label_on_clicked(self):
		# -------------------------------------
		# Init mode selection dictionary if it doesnt exist
		if self.mode_selection_id_min is None:
			self.init_mode_selection_dictionary()

		# Get selected mode
		if self.mode_selection_id_current is None:
			selected_mode = 0
		else:
			index = self.mode_selection_list.index(self.mode_selection_id_current)
			if index == self.mode_selection_id_min:
				selected_mode = self.mode_selection_id_max
			else:
				selected_mode = self.mode_selection_list[index - 1]

		# Set selected mode
		self.mode_selection_id_current = selected_mode
		self.mode_selection_label.set_text(self.mode_selection_dict[selected_mode]['name'])
		self.column_layout = COLUMN_LAYOUT_TEMPLATES_LEADERBOARD[self.mode_selection_id_current]['layout']

		# Update window
		self.update_window()

	# -------------------------------------
	def mode_selection_right_label_on_clicked(self):
		# -------------------------------------
		# Init mode selection dictionary if it doesnt exist
		if self.mode_selection_id_min is None:
			self.init_mode_selection_dictionary()

		# Get selected mode
		if self.mode_selection_id_current is None:
			selected_mode = 0
		else:
			index = self.mode_selection_list.index(self.mode_selection_id_current)
			if index == self.mode_selection_id_max:
				selected_mode = self.mode_selection_id_min
			else:
				selected_mode = self.mode_selection_list[index + 1]

		# Set selected mode
		self.mode_selection_id_current = selected_mode
		self.mode_selection_label.set_text(self.mode_selection_dict[selected_mode]['name'])
		self.column_layout = COLUMN_LAYOUT_TEMPLATES_LEADERBOARD[self.mode_selection_id_current]['layout']

		# Update window
		self.update_window()

# -------------------------------------
def mode_selection_left_label_on_clicked(*args):
	# -------------------------------------
	app_leaderboard.mode_selection_left_label_on_clicked()

# -------------------------------------
def mode_selection_right_label_on_clicked(*args):
	# -------------------------------------
	app_leaderboard.mode_selection_right_label_on_clicked()

# -------------------------------------
# Main
# -------------------------------------
app_leaderboard = AppLeaderboard()
