# !/usr/bin/env python
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Title       : config.py
# Author      : aDd1kTeD2Ka0s (add1kted2ka0s@gmail.com)
# Date        : 20-Jun-2014
# Description : Ka0s Leaderboard - Configuration
# -----------------------------------------------------------------------------

# -------------------------------------
# Imports
# -------------------------------------

import ac
from ..lib.colors import RGB

# -------------------------------------
# Global Variables
# !!! DO NOT MODIFY THIS !!!
# -------------------------------------

KM_TO_MILES = 1.60934
SPEED_UNIT_KMH = {'speed_suffix': 'kmh', 'distance_suffix_large': 'km', 'distance_suffix_small': 'm', 'distance_conversion_factor': 1000}
SPEED_UNIT_MPH = {'speed_suffix': 'mph', 'distance_suffix_large': 'mi', 'distance_suffix_small': 'yd', 'distance_conversion_factor': 1760}

# -------------------------------------
# User Settings
# -------------------------------------

# ----------- App Settings ------------
# Toggle whether to always include the leader [True|False]
INCLUDE_LEADER = True

# Number of rows to display in the app (excluding the header)
NUMBER_OF_ROWS_APP_LEADERBOARD = 8
NUMBER_OF_ROWS_APP_LAPS = 8

# Toggle whether to keep previous previous lap splits for some time [True|False]
# True  : Resets previous lap splits 'PREVIOUS_LAP_SPLITS_DECAY_TIME' milliseconds after starting a new lap
# False : Keeps previous lap splits until you reach that sector
PREVIOUS_LAP_SPLITS_DECAY = False

# Time period (milliseconds) to keep previous laps splits when PREVIOUS_LAP_SPLITS_DECAY = True
PREVIOUS_LAP_SPLITS_DECAY_TIME = 10000

# Speed unit of measurement [SPEED_UNIT_MPH|SPEED_UNIT_KMH]
SPEED_UNIT = SPEED_UNIT_KMH

# How often to refresh the app display
# --- Average framerate you get in-game
AVERAGE_FRAMERATE = 40
# --- Frames to skip before updating the app display
# --- Increase this for slower updates (e.g. if you have a slower PC)
# --- Decrease this for faster updates (0 for real-time)
UPDATE_EVERY_X_FRAMES = 2

# ---------- Window Settings ----------
WINDOW_OPACITY = 0.5
WINDOW_BORDER = 5 # Pixels
WINDOW_TOP_OFFSET = 0 # Pixels
WINDOW_PIN_WIDTH = 35 # Pixels

# ----------- Font Settings -----------
FONT_SIZE = 14
LETTER_WIDTH = FONT_SIZE / 2.0 # Approximate letter width. You would need to adjust the divisor if you change FONT_SIZE
ROW_HEIGHT = 16 # should be greater than FONT_SIZE

# -------------- Colors ---------------
# Color settings for header
COLOR_FOREGROUND_HEADER = RGB(255, 255, 255)
COLOR_BACKGROUND_HEADER = RGB(255, 0, 0)
OPACITY_BACKGROUND_HEADER = 0.2

# Color settings for mode selection text
COLOR_FOREGROUND_MODE_SELECTION_TEXT = RGB(255, 255, 255)
COLOR_BACKGROUND_MODE_SELECTION_TEXT = RGB(0, 0, 0)
OPACITY_BACKGROUND_MODE_SELECTION_TEXT = 0.0

# Color settings for mode selection selector
COLOR_FOREGROUND_MODE_SELECTION_SELECTOR = RGB(255, 255, 255)
COLOR_BACKGROUND_MODE_SELECTION_SELECTOR = RGB(127, 127, 127)
OPACITY_BACKGROUND_MODE_SELECTION_SELECTOR = 0.3

# Color settings for odd rows (1,3,5,7...)
COLOR_FOREGROUND_ROW_1 = RGB(255, 255, 255)
COLOR_BACKGROUND_ROW_1 = RGB(0, 0, 0)
OPACITY_BACKGROUND_ROW_1 = 0.3

# Color settings for even rows (2,4,6,8...)
COLOR_FOREGROUND_ROW_2 = RGB(255, 255, 255)
COLOR_BACKGROUND_ROW_2 = RGB(96, 96, 96)
OPACITY_BACKGROUND_ROW_2 = 0.3

# Enable self coloring [True|False]
ENABLE_SELF_COLOR = True

# Color settings for self
# Set these values to None if you don't want this to override the normal color for that cell
COLOR_FOREGROUND_SELF = RGB(255, 255, 255)
COLOR_BACKGROUND_SELF = RGB(192, 192, 0)
OPACITY_BACKGROUND_SELF = 0.3

# Enable best/worst coloring [True|False]
ENABLE_BEST_WORST_COLOR = True

# Color settings for best split/speed/lap time
# Set these values to None if you don't want this to override the normal color for that cell
COLOR_FOREGROUND_BEST = RGB(32, 255, 32)
COLOR_BACKGROUND_BEST = None
OPACITY_BACKGROUND_BEST = None

# Color settings for worst split/speed/lap time
COLOR_FOREGROUND_WORST = RGB(255, 32, 32)
COLOR_BACKGROUND_WORST = None
OPACITY_BACKGROUND_WORST = None

# -------------------------------------
# Global Variables
# !!! DO NOT MODIFY THIS !!!
# -------------------------------------

SPLIT_COUNT = len(ac.getCurrentSplits(0))

COLOR_BEST = {'foreground': COLOR_FOREGROUND_BEST, 'background': COLOR_BACKGROUND_BEST, 'opacity': OPACITY_BACKGROUND_BEST}
COLOR_WORST = {'foreground': COLOR_FOREGROUND_WORST, 'background': COLOR_BACKGROUND_WORST, 'opacity': OPACITY_BACKGROUND_WORST}

UPDATE_FREQUENCY = (1.0 / float(AVERAGE_FRAMERATE)) * float(UPDATE_EVERY_X_FRAMES)

# --- Column Settings (Leaderboard)----
# Defines the available columns
#	'<COLUMN_NAME>': {          # <COLUMN_NAME> = Identifier used in the code
#		'label': '<LABEL>',     # <LABEL> = Display label for column header
#		'width': <WIDTH>,       # <WIDTH> = Width of column (in characters, not pixels)
#		'align': '<ALIGNMENT>'  # <ALIGNMENT> = [left|center|right]
#	},
# Note : 1. You can _NOT_ create a new column without corresponding changes in the code
#      : 2. You can play around with the label,width and alignment fields to suit your needs
#      : 3. COL_SPLIT is a special column, it actually creates 1 column for every sector on the track
COL_SETTINGS_LEADERBOARD = {
	'COL_POSITION': {
		'label': '#',
		'width': 3,
		'align': 'left'
	},
	'COL_NAME': {
		'label': 'Name',
		'width': 18,
		'align': 'left'
	},
	'COL_CAR': {
		'label': 'Car',
		'width': 18,
		'align': 'left'
	},
	'COL_LAPS': {
		'label': 'Laps',
		'width': 6,
		'align': 'right'
	},
	'COL_COMPLETION_LAP': {
		'label': 'Lap %',
		'width': 9,
		'align': 'right'
	},
	'COL_COMPLETION_RACE': {
		'label': 'Race %',
		'width': 9,
		'align': 'right'
	},
	'COL_DISTANCE_LAP': {
		'label': 'Lap Dist',
		'width': 11,
		'align': 'right'
	},
	'COL_DISTANCE_RACE': {
		'label': 'Race Dist',
		'width': 11,
		'align': 'right'
	},
	'COL_DIFF_TIME_LEAD': {
		'label': '+Lead',
		'width': 10,
		'align': 'right'
	},
	'COL_DIFF_TIME_PREV': {
		'label': '+Prev',
		'width': 10,
		'align': 'right'
	},
	'COL_DIFF_TIME_YOU': {
		'label': '+You',
		'width': 10,
		'align': 'right'
	},
	'COL_DIFF_DISTANCE_LEAD': {
		'label': '+Lead',
		'width': 12,
		'align': 'right'
	},
	'COL_DIFF_DISTANCE_PREV': {
		'label': '+Prev',
		'width': 12,
		'align': 'right'
	},
	'COL_DIFF_DISTANCE_YOU': {
		'label': '+You',
		'width': 12,
		'align': 'right'
	},
	'COL_SPLIT': {
		'label': 'S',
		'width': 8,
		'align': 'right'
	},
	'COL_CURR_LAP': {
		'label': 'Curr',
		'width': 9,
		'align': 'right'
	},
	'COL_LAST_LAP': {
		'label': 'Last',
		'width': 9,
		'align': 'right'
	},
	'COL_BEST_LAP': {
		'label': 'Best',
		'width': 9,
		'align': 'right'
	},
	'COL_SPEED_CURRENT': {
		'label': 'Speed',
		'width': 10,
		'align': 'right'
	},
	'COL_SPEED_MAX': {
		'label': 'Top Speed',
		'width': 12,
		'align': 'right'
	}
}

# Defines which columns are shown and the order in which they are shown in the app window
COLUMN_LAYOUT_TEMPLATES_LEADERBOARD = {
	0: {
		'name': 'Default Layout',
		'layout': ['COL_POSITION', 'COL_NAME', 'COL_LAPS', 'COL_COMPLETION_LAP', 'COL_DIFF_TIME_LEAD', 'COL_DIFF_DISTANCE_LEAD', 'COL_SPLIT', 'COL_CURR_LAP', 'COL_LAST_LAP', 'COL_BEST_LAP']
	},
	1: {
		'name': 'Minimal Layout (T)',
		'layout': ['COL_POSITION', 'COL_NAME', 'COL_DIFF_TIME_LEAD']
	},
	2: {
		'name': 'Minimal Layout (D)',
		'layout': ['COL_POSITION', 'COL_NAME', 'COL_DIFF_DISTANCE_LEAD']
	},
	3: {
		'name': 'Minimal Layout (T+D)',
		'layout': ['COL_POSITION', 'COL_NAME', 'COL_DIFF_TIME_LEAD', 'COL_DIFF_DISTANCE_LEAD']
	},
	4: {
		'name': 'Delta Layout (T)',
		'layout': ['COL_POSITION', 'COL_NAME', 'COL_DIFF_TIME_LEAD', 'COL_DIFF_TIME_PREV', 'COL_DIFF_TIME_YOU']
	},
	5: {
		'name': 'Delta Layout (D)',
		'layout': ['COL_POSITION', 'COL_NAME', 'COL_DIFF_DISTANCE_LEAD', 'COL_DIFF_DISTANCE_PREV', 'COL_DIFF_DISTANCE_YOU']
	},
	6: {
		'name': 'Delta Layout (T+D)',
		'layout': ['COL_POSITION', 'COL_NAME', 'COL_DIFF_TIME_LEAD', 'COL_DIFF_TIME_PREV', 'COL_DIFF_TIME_YOU', 'COL_DIFF_DISTANCE_LEAD', 'COL_DIFF_DISTANCE_PREV', 'COL_DIFF_DISTANCE_YOU']
	},
	7: {
		'name': 'Completion Layout',
		'layout': ['COL_POSITION', 'COL_NAME', 'COL_LAPS', 'COL_COMPLETION_LAP', 'COL_DISTANCE_LAP', 'COL_COMPLETION_RACE', 'COL_DISTANCE_RACE']
	},
	8: {
		'name': 'Speed Layout',
		'layout': ['COL_POSITION', 'COL_NAME', 'COL_CAR', 'COL_SPEED_CURRENT', 'COL_SPEED_MAX']
	},
	9: {
		'name': 'Drag Race Layout',
		'layout': ['COL_POSITION', 'COL_NAME', 'COL_CURR_LAP', 'COL_DIFF_TIME_LEAD', 'COL_DIFF_DISTANCE_LEAD', 'COL_SPEED_CURRENT', 'COL_SPEED_MAX']
	},
	#8: {
	#	'name': 'All Columns Layout',
	#	'layout': ['COL_POSITION', 'COL_NAME', 'COL_CAR', 'COL_LAPS', 'COL_COMPLETION_LAP', 'COL_DISTANCE_LAP', 'COL_COMPLETION_RACE', 'COL_DISTANCE_RACE', 'COL_DIFF_TIME_LEAD', 'COL_DIFF_TIME_PREV', 'COL_DIFF_TIME_YOU', 'COL_DIFF_DISTANCE_LEAD', 'COL_DIFF_DISTANCE_PREV', 'COL_DIFF_DISTANCE_YOU', 'COL_SPLIT', 'COL_CURR_LAP', 'COL_LAST_LAP', 'COL_BEST_LAP', 'COL_SPEED_CURRENT', 'COL_SPEED_MAX']
	#}
}

# ------ Column Settings (Laps) -------
# Defines the available columns
#	'<COLUMN_NAME>': {          # <COLUMN_NAME> = Identifier used in the code
#		'label': '<LABEL>',     # <LABEL> = Display label for column header
#		'width': <WIDTH>,       # <WIDTH> = Width of column (in characters, not pixels)
#		'align': '<ALIGNMENT>'  # <ALIGNMENT> = [left|center|right]
#	},
# Note : 1. You can _NOT_ create a new column without corresponding changes in the code
#      : 2. You can play around with the label,width and alignment fields to suit your needs
#      : 3. COL_SPLIT is a special column, it actually creates 1 column for every sector on the track
COL_SETTINGS_LAPS = {
	'COL_LAP_NUM': {
		'label': '#',
		'width': 5,
		'align': 'left'
	},
	'COL_SPLIT': {
		'label': 'S',
		'width': 8,
		'align': 'right'
	},
	'COL_LAP_TIME': {
		'label': 'Lap Time',
		'width': 9,
		'align': 'right'
	}
}

# Defines which columns are shown and the order in which they are shown in the app window
COLUMN_LAYOUT_TEMPLATES_LAPS = {
	0: {
		'name': 'Default',
		'layout': ['COL_LAP_NUM', 'COL_SPLIT', 'COL_LAP_TIME']
	}
}
