# !/usr/bin/env python
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Title       : settings.py
# Author      : aDd1kTeD2Ka0s (add1kted2ka0s@gmail.com)
# Date        : 20-Jun-2014
# Description : Ka0s Leaderboard - Settings
#             : Original code by Rombik
# -----------------------------------------------------------------------------

# -------------------------------------
# Imports
# -------------------------------------

import contextlib
import json
import os
import shutil

# -------------------------------------
class Settings(dict):
	# -------------------------------------

	# -------------------------------------
	def __init__(self, filename, defaults=None, auto_save=False):
		# -------------------------------------
		super().__init__()
		self.auto_save = auto_save
		self.filename = filename
		self.update(defaults if defaults is not None else {})
		self.load()

	# -------------------------------------
	def load(self):
		# -------------------------------------
		try:
			with open(self.filename) as f:
				data = json.load(f)
			self.update(data)
		except Exception:
			pass  # I know it's bad

	# -------------------------------------
	def save(self):
		# -------------------------------------
		dirname = os.path.dirname(self.filename)
		if dirname:
			os.makedirs(dirname, exist_ok=True)
		with atomic_writer(self.filename) as f:
			json.dump(self, f)

	# -------------------------------------
	def __setitem__(self, key, value):
		# -------------------------------------
		super().__setitem__(key, value)
		if self.auto_save:
			self.save()

# -------------------------------------
@contextlib.contextmanager
def atomic_writer(filename, mode='w', suffix=u'.tmp$'):
	# -------------------------------------
	tmp_name = filename + suffix
	out = open(tmp_name, mode)
	try:
		yield out
		out.close()
		shutil.move(tmp_name, filename)  # Not really atomic
	except:
		raise
	finally:
		out.close()

# -------------------------------------
if __name__ == '__main__':
	# -------------------------------------
	s = Settings('d:/tmp/tst.json', {'b': 2})
	print(s.get('a'), s.get('b'))
	s['a'] = 1
	s.save()
	s = Settings('tst.json', auto_save=True)
	s['c'] = 3
	s = Settings('tst.json')
	print(s['c'])
